  /************************************************************************
   *                                                                      *
   *     REWRITE.C -- Rewrite file maintenence commands                   *
   *                                                                      *
   *     Added on 11/11/94 by Todd Powers, N8WEI                          *
   *                                                                      *
   *     These commands allow the sysop to maintain the rewrite file      *
   *     from the console, instead of having to shell out and edit it     *
   *     every time they wish to change something...                      *
   *                                                                      *
   *     They also allow the sysop to "debug" their rewrite file.         *
   *                                                                      *
   ************************************************************************/

#include "global.h"

#ifdef REWRITECMD

#include "rewrite.h"
#include "cmdparse.h"
#include "commands.h"
#include "files.h"
#include "mailutil.h"
#include "smtp.h"
#include "socket.h"

static struct cmds Rcmds[] = {
    "add",      dorewadd,              1024,    3, "add <address> <destination>",
    "delete",   dorewdel,              1024,    3, "delete <address> <destination>",
    "look",     dorewlook,              512,    2, "look <search text>",
    "resolve",  dorewres,               512,    2, "resolve <address> [[<address>]...]",
    "trace",    dorewtra,               512,    2, "trace <address> [[<address>]...]",
	NULLCHAR,
};
int
dorewrite(argc,argv,p)
int argc;
char *argv[];
void *p;
{
    int ret_val;
    ret_val = subcmd(Rcmds,argc,argv,p);
    usflush(Curproc->output);
    return ret_val;
}
int
dorewadd(argc,argv,p)
int argc;
char *argv[];
void *p;
{
    FILE *fp;
    FILE *fp2;
    char *buffer;
    int a;
    if((fp = fopen(Rewritefile,READ_TEXT)) == NULLFILE)
        return -1;     /* Userfile doesn't exist */
    if((fp2 = tmpfile()) == NULLFILE) {
		fclose(fp);
        return -1;     /* Unable to create temp file */
		}
	for(a=1;a<argc;a++)
        fprintf(fp2,"%s ",argv[a]);
	fprintf(fp2,"\n");
	while(fgets(buffer,256,fp),!feof(fp))
		fputs(buffer,fp2);
	fclose(fp);
	rewind(fp2);
    if((fp = fopen(Rewritefile,WRITE_TEXT)) == NULLFILE) {
        tprintf("\nUnable to open %s for writing...\n\n",Rewritefile);
		fclose(fp2);
        usflush(Curproc->output);
        return -1;     /* Unable to write to rewrite file */
		}
	while(fgets(buffer,256,fp2),!feof(fp2))
		fputs(buffer,fp);
	fclose(fp);
	fclose(fp2);
    usflush(Curproc->output);
    return 0;
}
int
dorewdel(argc,argv,p)
int argc;
char *argv[];
void *p;
{
    FILE *fp,*fp2;
    char *buffer,*string;
    int a,deleted;
    string = strdup(argv[1]);
    string = strcat(string," \0");
    for(a=2;a<argc;a++) {
        string = strcat(string,argv[a]);
        string = strcat(string," \0");
		}
    if((fp = fopen(Rewritefile,READ_TEXT)) == NULLFILE) {
        return -1;     /* Userfile doesn't exist */
		}
    if((fp2 = tmpfile()) == NULLFILE) {
		fclose(fp);
        return -1;     /* Unable to create temp file */
		}
    deleted = 0;
    while(fgets(buffer,256,fp),!feof(fp)) {
        if(strncmp(buffer,string,strlen(string)-1) != 0)
			fputs(buffer,fp2);
		else
            deleted = 1;
		}
	fclose(fp);
	rewind(fp2);
	free(string);
    if((fp = fopen(Rewritefile,WRITE_TEXT)) == NULLFILE) {
        tprintf("\nUnable to open %s for writing...\n\n",Rewritefile);
		fclose(fp2);
        usflush(Curproc->output);
        return -1;     /* Unable to write to rewrite file */
		}
	while(fgets(buffer,256,fp2),!feof(fp2))
		fputs(buffer,fp);
	fclose(fp);
	fclose(fp2);
    if(deleted == 0)
        tprintf("No match found...\n");
    usflush(Curproc->output);
    return 0;
}
int
dorewlook(argc,argv,p)
int argc;
char *argv[];
void *p;
{
    char **margv;
    margv = (char **)callocw(3,sizeof(char *));
    margv[1] = strdup(Rewritefile);
    margv[2] = strdup(argv[1]);
	domore(3,margv,p);
	free(margv[1]);
	free(margv[2]);
	free(margv);
    usflush(Curproc->output);
    return 0;
}
int
dorewres(argc,argv,p)
int argc;
char *argv[];
void *p;
{
    char *result;
    char address_type;
    int a;
    if(argc > 2)
		tprintf("\n");
    for(a=1;a<argc;a++) {
        /* rewrite address if possible */
        if((result = rewrite_address(argv[a])) == NULLCHAR) {
            tprintf("Local:     %s\n",argv[a]);
			free(result);
        } else {
            /* check if address is ok */
            if ((address_type = validate_address(result)) == 0) {
                tprintf("<%s> Address unknown\n",result);
				free(result);
            } else {
                if(address_type == 1)
                    tprintf("Local:     ");
                if(address_type == 2)
                    tprintf("Domain:    ");
                if(address_type == 3)
                    tprintf("NNTP Gate: ");
				tprintf("%s\n",result);
				free(result);
				}
			}
		}
    usflush(Curproc->output);
    return 0;
}
int
dorewtra(argc,argv,p)
int argc;
char *argv[];
void *p;
{
    char *result;
    char address_type;
    int a;
	tprintf("\n");
	for(a=1;a<argc;a++)
		{
        /* rewrite address if possible */
        if((result = rewrite_trace(argv[a])) == NULLCHAR)
			{
            tprintf("Local:     %s\n",argv[a]);
			free(result);
			}
		else
			{
            /* check if address is ok */
            if ((address_type = validate_address(result)) == 0)
				{
                tprintf("<%s> Address unknown\n",result);
				}
			else
				{
                if(address_type == 1)
                    tprintf("Local:     ");
                if(address_type == 2)
                    tprintf("Domain:    ");
                if(address_type == 3)
                    tprintf("NNTP Gate: ");
				tprintf("%s\n",result);
				free(result);
				}
			}
		}
    usflush(Curproc->output);
    return 0;
}
#endif /* REWRITECMD */
