  /************************************************************************
   *                                                                      *
   *     USERMNT.C -- User Maintenance commands                           *
   *                                                                      *
   *     Added on 11/10/94 by Todd Powers, N8WEI                          *
   *                                                                      *
   *     These commands allow the sysop to maintain the ftpusers file     *
   *     from the console, instead of having to shell out and edit it     *
   *     every time they wish to change something...                      *
   *                                                                      *
   ************************************************************************/

#include "global.h"
#include "usermnt.h"
#include "cmdparse.h"
#include "commands.h"
#include "files.h"
#include "session.h"
#include "socket.h"

#ifdef USERMAINT
#undef USERMAINT_DEBUG 1

static int UserRemote = 0;
static struct cmds UScmds[] = {
    "add",          douseradd,               1024,  3,  "add <username> <password>",
    "delete",       douserdel,               1024,  2,  "delete <username>",
    "list",         douserlist,                 0,  1,  NULLCHAR,
    "look",         douserlook,                 0,  2,  "look <username>",
    "password",     douserpass,              1024,  3,  "pass <username> <password>",
    "permission",   douserpermit,               0,  0,  NULLCHAR,
    "remote",       douserremote,               0,  1,  NULLCHAR,
    NULLCHAR,       NULLFP,                     0,  0,  NULLCHAR
};

static struct cmds PERMcmds[] = {
    "add",          douserpadd,              1024,  4,  "add <username> <path> <level>",
    "change",       douserpchg,              1024,  5,  "change <username> <old_path> <new_path> <level>",
    "delete",       douserpdel,              1024,  3,  "delete <username> <path>",
    NULLCHAR,       NULLFP,                     0,  0,  NULLCHAR
};

int
dousermaint(argc,argv,p)
int argc;
char *argv[];
void *p;
{
    int ret_val=0;

    if ( UserRemote == 1 || Curproc->input == Command->input ) {
        ret_val = subcmd(UScmds,argc,argv,p);
        usflush(Curproc->output);
        return ret_val;
    } else {
        tprintf("\aPermission Denied!  User Maintenence available from the console only!\n");
        usflush(Curproc->output);
        return -1;
    }
}

int
douserlist(argc,argv,p)
int argc;
char *argv[];
void *p;
{
    FILE *fp=NULL;
    char path[512],*user,*password,*directory,*level,*tmpstr;
    int a=0,b=0;

    if((fp = fopen(Userfile,READ_TEXT)) == NULLFILE)
        return -1;     /* Userfile doesn't exist */

    tprintf("\n");
    tprintf("Username        Password        Directory Privilages\n");
    tprintf("--------------- --------------- ---------------------------------------------\n");
    usflush(Curproc->output);
    while(fgets(path,512,fp),!feof(fp)) {
        a = 32;
        user = skipwhite(path);
        if(*user == '#')
            continue;     /* Comment */
        if((password = strpbrk(user," \t")) == NULLCHAR)
            continue;     /* Bogus entry */
        rip(user);
        *password++ = '\0';     /* Now points to spaces or tabs before password */
        password = skipwhite(password);     /* Now points to password */
        tprintf("%-15s ",user);
        if((directory = strpbrk(password," \t")) == NULLCHAR) {
            tprintf("%-15s ",password);
            tprintf("*** No permissions for this user ***\n");
            continue;     /* No permissions for this user */
        }
        *directory++ = '\0';
        directory = skipwhite(directory);
        tprintf("%-15s ",password);
        if(strlen(directory) < 1) {
            tprintf("*** No permissions for this user ***\n");
            continue;     /* No permissions for this user */
        }
        if((level = strpbrk(directory," \t")) == NULLCHAR) {
            tprintf("*** INVALID PERMISSION ENTRY ***\n");
            continue;     /* Bogus entry */
        }
        *level++ = '\0';
        level = skipwhite(level);
        if((tmpstr = strpbrk(level," \t")) != NULLCHAR) {
            *tmpstr++ = '\0';
            tmpstr = skipwhite(tmpstr);
        }
        while(strlen(directory) > 0)  {
            b = a + strlen(directory) + strlen(level) + 1;
            if(b >= 78 && a != 32) {
                b = 32;
                tprintf("\n");
                tprintf("                                ");
            }
            a=(b+1);
            tprintf("%s ",directory);
            tprintf("%s ",level);
            if(tmpstr == NULLCHAR) {
                tprintf("\n");
                break;
            } else {
                directory = strcpy(directory,tmpstr);
                if((level = strpbrk(directory," \t")) == NULLCHAR) {
                    tprintf("\n");
                    break;
                } else {
                    *level++ = '\0';
                    level = skipwhite(level);
                    if((tmpstr = strpbrk(level," \t")) != NULLCHAR) {
                        *tmpstr++ = '\0';
                        tmpstr = skipwhite(tmpstr);
                    }
                }
            }
        }
    }
    tprintf("\n");
    fclose(fp);
    usflush(Curproc->output);
    return 0;
}

int
douserlook(argc,argv,p)
int argc;
char *argv[];
void *p;
{
    FILE *fp=NULL;
    char path[512],*user,*password,*directory,*level;

    if((fp = fopen(Userfile,READ_TEXT)) == NULLFILE)
        return -1;     /* Userfile doesn't exist */

    while(fgets(path,512,fp),!feof(fp)) {
        user = skipwhite(path);
        if(*user == '#')
            continue;   /* Comment */
        if((password = strpbrk(user," \t")) == NULLCHAR)
            continue;   /* Bogus entry */
        rip(user);
        *password++ = '\0';     /* Now points to spaces or tabs before password */
        password = skipwhite(password);     /* Now points to password */
        if(stricmp(argv[1],user) == 0)
            break;      /* Found user name */
    }
    if(!feof(fp)) {
        if((directory = strpbrk(password," \t")) != NULLCHAR) {
            *directory++ = '\0';
            directory = skipwhite(directory);
            if((level = strpbrk(directory," \t")) != NULLCHAR) {
                *level++ = '\0';
                level = skipwhite(level);
            }
        }
        tprintf("\n");
        tprintf("Username: %s\n",user);
        tprintf("Password: %s\n",password);
        tprintf("Directory                               Level\n");
        tprintf("-------------------------------------  -------\n");
        usflush(Curproc->output);
        if(strlen(directory) < 1)
            tprintf("*** No permissions for this user ***\n");
        while(strlen(directory) > 0) {
            tprintf("%-37s",directory);
            if((directory = strpbrk(level," \t")) == NULLCHAR) {
                tprintf("  %6s\n",level);
                break;
            } else {
                *directory++ = '\0';
                tprintf("  %6s\n",level);
                directory = skipwhite(directory);
                if((level = strpbrk(directory," \t")) == NULLCHAR)
                    break;
                else {
                    *level++ = '\0';
                    level = skipwhite(level);
                }
            }
        }
        tprintf("\n");
    } else {
        tprintf("User \"%s\" not found...\n",argv[1]);
    }
    fclose(fp);
    usflush(Curproc->output);
    return 0;
}

int
douseradd(argc,argv,p)
int argc;
char *argv[];
void *p;
{
    FILE *fp=NULL,*fp2=NULL,*fp3=NULL;
    int ufound=0;
    char *path,*user,*password,*directory,*perms;

    if((fp2 = tmpfile()) != NULLFILE) {
        if((fp = fopen(Userfile,READ_TEXT)) == NULLFILE) {
            fclose(fp2);
            return -1;     /* Userfile doesn't exist */
        }
        while(fgets(path,512,fp),!feof(fp)) {
            user = skipwhite(path);
            if(*user == '#') {
                fprintf(fp2,"%s",user);
                continue;   /* Comment */
            }
            if((password = strpbrk(user," \t")) == NULLCHAR) {
                fprintf(fp2,"%s",user);
                continue;     /* Bogus entry */
            }
            rip(user);
            *password++ = '\0';     /* Now points to spaces or tabs before password */
            password = skipwhite(password);     /* Now points to password */
            if((directory = strpbrk(password," \t")) == NULLCHAR) {
                fprintf(fp2,"%-15s %s\n",user,password);
                continue;     /* Bogus entry */
            }
            *directory++ = '\0';
            directory = skipwhite(directory);
            fprintf(fp2,"%-15s %-15s %s\n",user,password,directory);
            if(strcmp(user,argv[1]) == 0)
                ufound = 1;
        }
        fclose(fp);
        if(!ufound) {
            if((fp = fopen(InitPerms,READ_TEXT)) == NULLFILE) {
                tprintf("Unable to open \"%s\"!\a\n",InitPerms);
                tprintf("This user will be added, but will have no permissions...\n");
                tprintf("Use the 'user permission add' command to add permissions for this user.\n");
                fprintf(fp2,"%-15s %-15s\n",argv[1],argv[2]);
                usflush(Curproc->output);
            } else {
                perms = (char *) malloc(512);
                perms[0] = '\0';
                while(fgets(path,512,fp),!feof(fp)) {
                    if(*path == '#')
                        continue;  /* Comment */
                    rip(path);
                    path = skipwhite(path);
                    perms = strcat(perms,path);
                    perms = strcat(perms," \0");
                }
                fprintf(fp2,"%-15s %-15s %s\n",argv[1],argv[2],perms);
                free(perms);
                fclose(fp);
            }
            rewind(fp2);
            if((fp = fopen(Userfile,WRITE_TEXT)) == NULLFILE) {
                tprintf("Unable to change %s...",Userfile);
                fclose(fp2);
                usflush(Curproc->output);
                return -1;     /* Userfile cannot be overwirtten */
            }
            while(fgets(path,512,fp2),!feof(fp2))
                fprintf(fp,"%s",path);
            fclose(fp);
            fclose(fp2);
            tprintf("\n");
            tprintf("User \"%s\" created...\n",argv[1]);
        } else {
            fclose(fp2);
            tprintf("\n");
            tprintf("\aUser \"%s\" already exists!\n",argv[1]);
        }
        usflush(Curproc->output);
        return 0;
    } else {
        tprintf("\n");
        tprintf("Unable to create temp file...\n");
        usflush(Curproc->output);
        return -1;
    }
}

int
douserdel(argc,argv,p)
int argc;
char *argv[];
void *p;
{
    FILE *fp=NULL,*fp2=NULL;
    int udeleted=0;
    char path[512],*user,*password,*directory;

    if((fp2 = tmpfile()) != NULLFILE) {
        if((fp = fopen(Userfile,READ_TEXT)) == NULLFILE) {
            fclose(fp2);
            return -1;     /* Userfile doesn't exist */
        }
        while(fgets(path,512,fp),!feof(fp)) {
            user = skipwhite(path);
            if(*user == '#') {
                fprintf(fp2,"%s",path);
                continue;   /* Comment */
            }
            if((password = strpbrk(user," \t")) == NULLCHAR) {
                fprintf(fp2,"%s",path);
                continue;     /* Bogus entry */
            }
            rip(user);
            *password++ = '\0';     /* Now points to spaces or tabs before password */
            password = skipwhite(password);     /* Now points to password */
            if(strcmp(user,argv[1]) != 0) {
                fprintf(fp2,"%-15s %s\n",user,password);
            } else {
                udeleted = 1;
            }
        }
        fclose(fp);
        if(!udeleted) {
            tprintf("\n");
            tprintf("No Such User...\n");
            usflush(Curproc->output);
            fclose(fp2);
        } else {
            rewind(fp2);
            if((fp = fopen(Userfile,WRITE_TEXT)) == NULLFILE) {
                tprintf("Unable to change %s...",Userfile);
                fclose(fp2);
                usflush(Curproc->output);
                return -1;     /* Userfile cannot be overwirtten */
            }
            while(fgets(path,512,fp2),!feof(fp2))
                fprintf(fp,"%s",path);
            fclose(fp);
            fclose(fp2);
            tprintf("\n");
            tprintf("\aUser \"%s\" deleted!\n",argv[1]);
        }
        usflush(Curproc->output);
        return 0;
    } else {
        tprintf("\n");
        tprintf("Unable to create temp file...\n");
        usflush(Curproc->output);
        return -1;
    }
}

int
douserpermit(argc,argv,p)
int argc;
char *argv[];
void *p;
{
    int ret_val=0;

    ret_val = subcmd(PERMcmds,argc,argv,p);
    usflush(Curproc->output);
    return ret_val;
}



int
douserpass(argc,argv,p)
int argc;
char *argv[];
void *p;
{
    FILE *fp=NULL,*fp2=NULL;
    char path[512],*user,*password,*directory,*level;
    int uchanged=0;

    if((fp2 = tmpfile()) != NULLFILE) {
        if((fp = fopen(Userfile,READ_TEXT)) == NULLFILE) {
            fclose(fp2);
            return -1;     /* Userfile doesn't exist */
        }
        while(fgets(path,512,fp),!feof(fp)) {
            user = skipwhite(path);
            if(*user == '#') {
                fprintf(fp2,"%s",path);
                continue;   /* Comment */
            }
            if((password = strpbrk(user," \t")) == NULLCHAR) {
                fprintf(fp2,"%s",path);
                continue;   /* Bogus entry */
            }
            rip(user);
            *password++ = '\0';     /* Now points to spaces or tabs before password */
            password = skipwhite(password);     /* Now points to password */
            if((directory = strpbrk(password," \t")) == NULLCHAR) {
                fprintf(fp2,"%-15s %s\n",user,password);
                continue;     /* Bogus entry */
            }
            *directory++ = '\0';    /* Now points to spaces or tabs before directory */
            directory = skipwhite(directory);
            if((level = strpbrk(directory," \t")) == NULLCHAR) {
                fprintf(fp2,"%-15s %-15s %s\n",user,password,directory);
                continue;     /* Bogus entry */
            }
            *level++ = '\0';
            level = skipwhite(level);
            if(stricmp(argv[1],user) == 0) {
                fprintf(fp2,"%-15s %-15s %s %s\n",user,argv[2],directory,level);
                tprintf("\n");
                tprintf("User \"%s\", password changed to \"%s\"\n",argv[1],argv[2]);
                uchanged = 1;
                usflush(Curproc->output);
            } else
                fprintf(fp2,"%-15s %-15s %s %s\n",user,password,directory,level);
        }
        fclose(fp);
        if(!uchanged) {
            tprintf("\n");
            tprintf("User \"%s\" not found!\n",argv[1]);
            fclose(fp2);
            usflush(Curproc->output);
        } else {
            rewind(fp2);
            if((fp = fopen(Userfile,WRITE_TEXT)) == NULLFILE) {
                tprintf("Unable to change %s...",Userfile);
                fclose(fp2);
                usflush(Curproc->output);
                return -1;     /* Userfile cannot be overwirtten */
            }
            while(fgets(path,512,fp2),!feof(fp2))
                fprintf(fp,"%s",path);
            fclose(fp);
            fclose(fp2);
        }
        usflush(Curproc->output);
        return 0;
    } else {
        tprintf("\n");
        tprintf("Unable to create temp file...\n");
        usflush(Curproc->output);
        return -1;
    }
}

int
douserremote(argc,argv,p)
int argc;
char *argv[];
void *p;
{
    return setbool( &UserRemote, "Allow Remote-Sysop access to the User-Maintenence commands.", argc,argv );
}


int
douserpadd(argc,argv,p)
int argc;
char *argv[];
void *p;
{
    FILE *fp=NULL,*fp2=NULL;
    int a=0,found=0,changed=0,added=0;
    char path[512],*user,*password,*directory,*level,*line;

    if((fp2 = tmpfile()) != NULLFILE) {
        if((fp = fopen(Userfile,READ_TEXT)) == NULLFILE) {
            fclose(fp2);
            return -1;     /* Userfile doesn't exist */
        }
        while(fgets(path,512,fp),!feof(fp)) {
            user = skipwhite(path);
            if(*user == '#') {
                fprintf(fp2,"%s",path);
                continue;   /* Comment */
            }
            if((password = strpbrk(user," \t")) == NULLCHAR) {
                fprintf(fp2,"%s",path);
                continue;     /* Bogus entry */
            }
            rip(user);
            *password++ = '\0';     /* Now points to spaces or tabs before password */
            password = skipwhite(password);     /* Now points to password */
            if(strcmp(user,argv[1]) != 0) {
                fprintf(fp2,"%-15s %s\n",user,password);
                continue;
            } else {
                found = 1;
                fprintf(fp2,"%-15s %s %s %s\n",user,password,argv[2],argv[3]);
                continue;
            }
        }
        fclose(fp);
        rewind(fp2);
        if((fp = fopen(Userfile,WRITE_TEXT)) == NULLFILE) {
            fclose(fp2);
            return -1;     /* Userfile doesn't exist or is write protected */
        }
        while(fgets(path,512,fp2),!feof(fp2))
            fputs(path,fp);
        fclose(fp);
        fclose(fp2);
    } else {
        tprintf("\n");
        tprintf("Unable to create temp file...\n");
        usflush(Curproc->output);
        return -1;
    }
    if(!found) {
        tprintf("\n");
        tprintf("User \"%s\" does not exist!\n",argv[1]);
        usflush(Curproc->output);
    } else {
        tprintf("\n");
        tprintf("User \"%s\" permission level \"%s %s\" added!\n",argv[1],argv[2],argv[3]);
        usflush(Curproc->output);
    }
    return 0;
}


int
douserpchg(argc,argv,p)
int argc;
char *argv[];
void *p;
{
    FILE *fp=NULL,*fp2=NULL;
    int a=0,found=0,changed=0,added=0;
    char path[512],*user,*password,*directory,*level,*line;

    if((fp2 = tmpfile()) != NULLFILE) {
        if((fp = fopen(Userfile,READ_TEXT)) == NULLFILE) {
            fclose(fp2);
            return -1;     /* Userfile doesn't exist */
        }
        while(fgets(path,512,fp),!feof(fp)) {
            user = skipwhite(path);
            if(*user == '#') {
                fprintf(fp2,"%s",path);
                continue;   /* Comment */
            }
            if((password = strpbrk(user," \t")) == NULLCHAR) {
                fprintf(fp2,"%s",path);
                continue;     /* Bogus entry */
            }
            rip(user);
            *password++ = '\0';     /* Now points to spaces or tabs before password */
            password = skipwhite(password);     /* Now points to password */
            if((directory = strpbrk(password," \t")) == NULLCHAR) {
                fprintf(fp2,"%-15s %s\n",user,password);
                continue;     /* Bogus entry */
            }
            *directory++ = '\0';
            directory = skipwhite(directory);
            if((level = strpbrk(directory," \t")) == NULLCHAR) {
                fprintf(fp2,"%-15s %-15s %s\n",user,password,directory);
                continue;
            }
            *level++ = '\0';
            level = skipwhite(level);
            if(strcmp(user,argv[1]) != 0) {
                fprintf(fp2,"%-15s %-15s %s %s\n",user,password,directory,level);
            } else {
                found = 1;
                fprintf(fp2,"%-15s %-15s",user,password);
                while(directory != NULLCHAR) {
                    if(stricmp(directory,argv[2]) == 0) {
                        changed=1;
                        fprintf(fp2," %s",argv[3]);
                        fprintf(fp2," %s",argv[4]);
                        if((directory = strpbrk(level," \t")) != NULLCHAR) {
                            *directory++ = '\0';
                            directory = skipwhite(directory);
                        }
                    } else {
                        fprintf(fp2," %s",directory);
                        if((directory = strpbrk(level," \t")) != NULLCHAR) {
                            *directory++ = '\0';
                            directory = skipwhite(directory);
                        }
                        fprintf(fp2," %s",level);
                    }
                    if(directory != NULLCHAR) {
                        if((level = strpbrk(directory," \t")) != NULLCHAR) {
                            *level++ = '\0';
                            level = skipwhite(level);
                            continue;
                        } else
                            directory = NULLCHAR;
                    }
                }
                fprintf(fp2,"\n");
                continue;
            }
        }
        fclose(fp);
        rewind(fp2);
        if((fp = fopen(Userfile,WRITE_TEXT)) == NULLFILE) {
            fclose(fp2);
            return -1;     /* Userfile doesn't exist or is write protected */
        }
        while(fgets(path,512,fp2),!feof(fp2))
            fputs(path,fp);
        fclose(fp);
        fclose(fp2);
    } else {
        tprintf("\n");
        tprintf("Unable to create temp file...\n");
        usflush(Curproc->output);
        return -1;
    }
    if(!found) {
        tprintf("\n");
        tprintf("User \"%s\" does not exist!\n",argv[1]);
        usflush(Curproc->output);
    } else {
        if(!changed) {
            tprintf("\n");
            tprintf("Unable to change permission level for user \"%s\"...\n",argv[1]);
            usflush(Curproc->output);
        } else {
            tprintf("\n");
            tprintf("User \"%s\" - Permissions for \"%s\"\n",argv[1],argv[2]);
            tprintf("changed to \"%s %s\"\n",argv[3],argv[4]);
            usflush(Curproc->output);
        }
    }
    return 0;
}


int
douserpdel(argc,argv,p)
int argc;
char *argv[];
void *p;
{
    FILE *fp=NULL,*fp2=NULL;
    int a=0,found=0,changed=0,added=0;
    char path[512],*user,*password,*directory,*level,*line;

    if((fp2 = tmpfile()) != NULLFILE) {
        if((fp = fopen(Userfile,READ_TEXT)) == NULLFILE) {
            fclose(fp2);
            return -1;     /* Userfile doesn't exist */
        }
        while(fgets(path,512,fp),!feof(fp)) {
            user = skipwhite(path);
            if(*user == '#') {
                fprintf(fp2,"%s",path);
                continue;   /* Comment */
            }
            if((password = strpbrk(user," \t")) == NULLCHAR) {
                fprintf(fp2,"%s",path);
                continue;     /* Bogus entry */
            }
            rip(user);
            *password++ = '\0';     /* Now points to spaces or tabs before password */
            password = skipwhite(password);     /* Now points to password */
            if((directory = strpbrk(password," \t")) == NULLCHAR) {
                fprintf(fp2,"%-15s %s\n",user,password);
                continue;     /* Bogus entry */
            }
            *directory++ = '\0';
            directory = skipwhite(directory);
            if((level = strpbrk(directory," \t")) == NULLCHAR) {
                fprintf(fp2,"%-15s %-15s %s\n",user,password,directory);
                continue;
            }
            *level++ = '\0';
            level = skipwhite(level);
            if(strcmp(user,argv[1]) != 0) {
                fprintf(fp2,"%-15s %-15s %s %s\n",user,password,directory,level);
            } else {
                found = 1;
                fprintf(fp2,"%-15s %-15s",user,password);
                while(directory != NULLCHAR) {
                    if(stricmp(directory,argv[2]) == 0) {
                        changed=1;
                        if((directory = strpbrk(level," \t")) != NULLCHAR) {
                            *directory++ = '\0';
                            directory = skipwhite(directory);
                        }
                    } else {
                        fprintf(fp2," %s",directory);
                        if((directory = strpbrk(level," \t")) != NULLCHAR) {
                            *directory++ = '\0';
                            directory = skipwhite(directory);
                        }
                        fprintf(fp2," %s",level);
                    }
                    if(directory != NULLCHAR) {
                        if((level = strpbrk(directory," \t")) != NULLCHAR) {
                            *level++ = '\0';
                            level = skipwhite(level);
                            continue;
                        } else
                            directory = NULLCHAR;
                    }
                }
                fprintf(fp2,"\n");
                continue;
            }
        }
        fclose(fp);
        rewind(fp2);
        if((fp = fopen(Userfile,WRITE_TEXT)) == NULLFILE) {
            fclose(fp2);
            return -1;     /* Userfile doesn't exist or is write protected */
        }
        while(fgets(path,512,fp2),!feof(fp2))
            fputs(path,fp);
        fclose(fp);
        fclose(fp2);
    } else {
        tprintf("\n");
        tprintf("Unable to create temp file...\n");
        usflush(Curproc->output);
        return -1;
    }
    if(!found) {
        tprintf("\n");
        tprintf("User \"%s\" does not exist!\n",argv[1]);
        usflush(Curproc->output);
    } else {
        if(!changed) {
            tprintf("\n");
            tprintf("Path \"%s\" does not exist for user \"%s\"...\n",argv[2],argv[1]);
            usflush(Curproc->output);
        } else {
            tprintf("\n");
            tprintf("Permission for \"%s\" deleted!\a\n",argv[2]);
            usflush(Curproc->output);
        }
    }
    return 0;
}


#endif /* USERMAINT */

